class CapmonsterException(Exception):
    def __init__(self, error_id, error_code, error_description, *args):
        super(CapmonsterException, self).__init__("[{}:{}]{}".format(error_code, error_id, error_description))
        self.error_description = error_description
        self.error_id = error_id
        self.error_code = error_code



from .exceptions import CapmonsterException
from .ImageToTextTask import ImageToTextTask
from .NoCaptchaTaskProxyless import NoCaptchaTaskProxyless
from .RecaptchaV3TaskProxyless import RecaptchaV3TaskProxyless
from .FunCaptchaTask import FunCaptchaTask
from .FunCaptchaTaskProxyless import FunCaptchaTaskProxyless
from .NoCaptchaTask import NoCaptchaTask
from .HCaptchaTask import HCaptchaTask
from .HCaptchaTaskProxyless import HCaptchaTaskProxyless


from .CapmonsterClient import CapmonsterClient
from .exceptions import CapmonsterException
import time


class FunCaptchaTaskProxyless(CapmonsterClient):
    def __init__(self, client_key, **kwargs):
        super().__init__(client_key=client_key, **kwargs)

    def createTask(self, website_url, website_public_key, js_subdomain=None, data_blob=None):
        data = {
            "clientKey": self.client_key,
            "task": {
                "type": "FunCaptchaTaskProxyless",
                "websiteURL": website_url,
                "websitePublicKey": website_public_key
            }
        }
        if js_subdomain is not None: data["task"]["funcaptchaApiJSSubdomain"] = js_subdomain
        if data_blob is not None: data["task"]["data"] = data_blob
        task = self.make_request(method="createTask", data=data)
        self.checkResponse(response=task)
        return task.json()["taskId"]

    def getTaskResult(self, taskId):
        data = {
            "clientKey": self.client_key,
            "taskId": taskId
        }
        task_result = self.make_request(method="getTaskResult", data=data)
        self.checkResponse(response=task_result)
        is_ready = self.checkReady(response=task_result)
        task_result = task_result.json()
        if is_ready:
            return task_result.get("solution").get("token")
        else:
            return False

    def joinTaskResult(self, taskId, maximum_time=120):
        data = {
            "clientKey": self.client_key,
            "taskId": taskId
        }
        i = 0
        while True:
            task_result = self.make_request(method="getTaskResult", data=data)
            self.checkResponse(response=task_result)
            is_ready = self.checkReady(response=task_result)
            task_result = task_result.json()
            if is_ready and task_result.get("solution") is not None:
                return task_result.get("solution").get("token")
            elif i >= maximum_time:
                raise CapmonsterException(None, 61, "Maximum time is exceed.")
            elif task_result.get("solution") is not None:
                i += 1
                time.sleep(2)
                continue



from .exceptions import CapmonsterException
from .CapmonsterClient import CapmonsterClient
import time


class NoCaptchaTaskProxyless(CapmonsterClient):
    def __init__(self, client_key, **kwargs):
        super().__init__(client_key=client_key, **kwargs)
        self.userAgent = kwargs.get("userAgent")

    def createTask(self, website_url, website_key, cookies=None):
        data = {
            "clientKey": self.client_key,
            "task":
                {
                    "type": "NoCaptchaTaskProxyless",
                    "websiteURL": website_url,
                    "websiteKey": website_key,
                    "userAgent": self.userAgent
                }
        }
        if cookies is not None and type(cookies) == dict or type(cookies) == list:
            add_cookies = ""
            if type(cookies) == dict:
                for key, value in cookies.items():
                    if value == list(cookies.items())[-1][1]:
                        add_cookies += "{}={}".format(key, value)
                    else:
                        add_cookies += "{}={};".format(key, value)
            elif type(cookies) == list:
                for i in cookies:
                    if not len(cookies) % 2 == 0:
                        raise AttributeError("List cookies length must be even numbers")
                    if cookies.index(i) % 2 == 0:
                        add_cookies += "{}=".format(i)
                    elif cookies[cookies.index(i)] == cookies[-1]:
                        add_cookies += "{}".format(i)
                    elif cookies.index(i) % 2 == 1:
                        add_cookies += "{};".format(i)
            data["task"]["cookies"] = add_cookies
            del add_cookies
        task = self.make_request(method="createTask", data=data)
        self.checkResponse(response=task)
        return task.json()["taskId"]

    def getTaskResult(self, taskId):
        data = {
            "clientKey": self.client_key,
            "taskId": taskId
        }
        task_result = self.make_request(method="getTaskResult", data=data)
        self.checkResponse(response=task_result)
        task_result = task_result.json()
        is_ready = self.checkReady(response=task_result)
        if is_ready:
            return task_result.get("solution").get("gRecaptchaResponse")
        else:
            return False

    def joinTaskResult(self, taskId, maximum_time=120):
        data = {
            "clientKey": self.client_key,
            "taskId": taskId
        }
        i = 0
        while True:
            task_result = self.make_request(method="getTaskResult", data=data)
            self.checkResponse(response=task_result)
            is_ready = self.checkReady(response=task_result)
            task_result = task_result.json()
            if is_ready and task_result.get("solution") is not None:
                return task_result.get("solution").get("gRecaptchaResponse")
            elif i >= maximum_time:
                raise CapmonsterException(None, 61, "Maximum time is exceed.")
            elif task_result.get("solution") is not None:
                i += 1
                time.sleep(2)
                continue



from .CapmonsterClient import CapmonsterClient
from .exceptions import CapmonsterException
from base64 import b64encode
import time


class ImageToTextTask(CapmonsterClient):
    def __init__(self, client_key):
        super().__init__(client_key=client_key)

    def createTask(self, file_path=None, base64_image: bytes = None, module=None):
        if file_path is None and base64_image is None:
            return False
        elif file_path is not None:
            image = open(file_path, "rb")
            img_base64 = b64encode(image.read()).decode("ascii")
        elif base64_image is not None:
            img_base64 = base64_image
        else:
            return False
        data = {
            "clientKey": self.client_key,
            "task":
            {
                "type": "ImageToTextTask",
                "body": img_base64
            }
        }
        if module is not None: data["task"]["СapMonsterModule"] = module
        task = self.make_request(method="createTask", data=data)
        self.checkResponse(response=task)
        return task.json().get("taskId")

    def getTaskResult(self, taskId):
        data = {
            "clientKey": self.client_key,
            "taskId": taskId
        }
        task_result = self.make_request(method="getTaskResult", data=data)
        self.checkResponse(response=task_result)
        is_ready = self.checkReady(response=task_result)
        task_result = task_result.json()
        if is_ready:
            return task_result.get("solution").get("text")
        else:
            return False

    def joinTaskResult(self, taskId, maximum_time=120):
        data = {
            "clientKey": self.client_key,
            "taskId": taskId
        }
        i = 0
        while True:
            task_result = self.make_request(method="getTaskResult", data=data)
            self.checkResponse(response=task_result)
            is_ready = self.checkReady(response=task_result)
            task_result = task_result.json()
            if is_ready and task_result.get("solution") is not None:
                return task_result.get("solution").get("text")
            elif i >= maximum_time:
                raise CapmonsterException(None, 61, "Maximum time is exceed.")
            elif task_result.get("solution") is not None:
                i += 1
                time.sleep(2)
                continue



from .CapmonsterClient import CapmonsterClient
from .exceptions import CapmonsterException
import time


class RecaptchaV3TaskProxyless(CapmonsterClient):
    def __init__(self, client_key, **kwargs):
        super().__init__(client_key=client_key, **kwargs)

    def createTask(self, website_url, website_key, minimum_score=0.3, page_action="verify"):
        if not (0.9 >= minimum_score >= 0.1):
            raise CapmonsterException(None, 99, "Minimum score must be between 0.1 and 0.9")
        data = {
            "clientKey": self.client_key,
            "task":
                {
                    "type": "RecaptchaV3TaskProxyless",
                    "websiteURL": website_url,
                    "websiteKey": website_key,
                    "minScore": minimum_score,
                    "pageAction": page_action
                }
        }
        task = self.make_request(method="createTask", data=data)
        self.checkResponse(response=task)
        return task.json().get("taskId")

    def getTaskResult(self, taskId):
        data = {
            "clientKey": self.client_key,
            "taskId": taskId
        }
        task_result = self.make_request(method="getTaskResult", data=data)
        self.checkResponse(response=task_result)
        is_ready = self.checkReady(response=task_result)
        task_result = task_result.json()
        if is_ready:
            return task_result.get("solution").get("gRecaptchaResponse")
        else:
            return False

    def joinTaskResult(self, taskId, maximum_time=120):
        data = {
            "clientKey": self.client_key,
            "taskId": taskId
        }
        i = 0
        while True:
            task_result = self.make_request(method="getTaskResult", data=data)
            self.checkResponse(response=task_result)
            is_ready = self.checkReady(response=task_result)
            task_result = task_result.json()
            if is_ready and task_result.get("solution") is not None:
                return task_result.get("solution").get("gRecaptchaResponse")
            elif i >= maximum_time:
                raise CapmonsterException(None, 61, "Maximum time is exceed.")
            elif task_result.get("solution") is not None:
                i += 1
                time.sleep(2)
                continue



import requests
from .exceptions import *


class CapmonsterClient(object):
    SOFT_ID = 30
    BALANCE_URL = "/getBalance"
    TASK_RESULT_URL = "/getTaskResult"
    CREATE_TASK_URL = "/createTask"

    def __init__(self, client_key, **kwargs):
        self.protocol = "https" if kwargs.get("protocol") == "https" else "http"
        self.client_key = client_key
        self.host = "{protocol}://api.capmonster.cloud".format(protocol=self.protocol)
        self.checkResponse(requests.post(url=f"{self.host}{self.BALANCE_URL}", json={"clientKey": self.client_key}))
        self.session = requests.Session()

    def checkResponse(self, response):
        response_json = response.json()
        if response_json.get("errorId") is not None and response_json.get("errorId") == 0:
            return True
        elif not response.status_code == 200:
            raise CapmonsterException(error_id=-1,
                                      error_code="HTTP_ERROR",
                                      error_description="Sometimes can be happen if capmonster servers there is too much intensity")
        elif not response_json.get("errorId") == 0:
            raise CapmonsterException(error_id=response_json.get("errorId"),
                                      error_code=response_json.get("errorCode"),
                                      error_description=response_json.get("errorDescription"))
        else:
            raise CapmonsterException(error_id=-1,
                                      error_code="HTTP_ERROR",
                                      error_description="Sometimes can be happen if capmonster servers there is too much intensity")

    def checkReady(self, response):
        status = response.json().get("status")
        if status == "processing":
            return False
        elif status == "ready":
            return True
        else:
            raise CapmonsterException(error_id=7,
                                      error_code="ERROR_NO_SUCH_CAPCHA_ID",
                                      error_description="The captcha that you are requesting was not found. Make sure you are requesting a status update only within 5 minutes of uploading.")

    def make_request(self, method, data):
        if method == "getBalance":
            method = self.BALANCE_URL
        elif method == "getTaskResult":
            method = self.TASK_RESULT_URL
        elif method == "createTask":
            method = self.CREATE_TASK_URL
            data["softId"] = self.SOFT_ID
        try:
            response = self.session.post(f"{self.host}{method}", json=data)
        except Exception as err:
            raise CapmonsterException(error_id=-1,
                                      error_code=type(err).__name__,
                                      error_description=str(err))
        return response

    def getBalance(self):
        data = {
            "clientKey": self.client_key
        }
        response = self.make_request(method="getBalance", data=data)
        self.checkResponse(response=response)
        return response.json().get("balance")



from distutils.core import setup

setup(
    name="capmonster_python",
    version="1.3.0",
    packages=["capmonster_python"],
    url="https://github.com/alperensert/capmonster_python",
    download_url="https://github.com/alperensert/capmonster_python/archive/v1.3.0.tar.gz",
    license="MIT",
    author="Alperen Sert",
    author_email="alperenssrt@gmail.com",
    description="Unofficial capmonster.cloud library for Python",
    requires=["requests"],
    classifiers=["Programming Language :: Python :: 3.4"]
)